@@ -2,7 +2,7 @@
  *
  *  HashKit library
  *
- *  Copyright (C) 2011 Data Differential, http://datadifferential.com/
+ *  Copyright (C) 2011-2012 Data Differential, http://datadifferential.com/
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are
@@ -37,7 +37,7 @@
 
 #pragma once
 
-#include <config.h>
+#include "libhashkit/hashkitcon.h"
 
 #include <assert.h>
 #include <errno.h>
@@ -45,19 +45,24 @@
 #include <stdlib.h>
 #include <math.h>
 
+#ifndef __WORDSIZE
+# ifdef __MINGW32__
+#  define __WORDSIZE 32
+# endif
+#endif
+
 #include <libhashkit-1.0/hashkit.h>
-#include <libhashkit/is.h>
-#include <libhashkit/string.h>
-#include <libhashkit/aes.h>
+#include "libhashkit/algorithm.h"
+#include "libhashkit/is.h"
+#include "libhashkit/string.h"
+#include "libhashkit/aes.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-HASHKIT_LOCAL
 void md5_signature(const unsigned char *key, unsigned int length, unsigned char *result);
 
-HASHKIT_LOCAL
 int update_continuum(hashkit_st *hashkit);
 
 #ifdef __cplusplus
