@@ -40,7 +40,7 @@
 */
 #include <libmemcached/common.h>
 
-static inline void _server_init(memcached_server_st *self, memcached_st *root,
+static inline void _server_init(memcached_server_st *self, Memcached *root,
                                 const memcached_string_t& hostname,
                                 in_port_t port,
                                 uint32_t weight, memcached_connection_t type)
@@ -54,6 +54,8 @@ static inline void _server_init(memcached_server_st *self, memcached_st *root,
   self->request_id= 0;
   self->server_failure_counter= 0;
   self->server_failure_counter_query_id= 0;
+  self->server_timeout_counter= 0;
+  self->server_timeout_counter_query_id= 0;
   self->weight= weight ? weight : 1; // 1 is the default weight value
   self->io_wait_count.read= 0;
   self->io_wait_count.write= 0;
@@ -82,7 +84,7 @@ static inline void _server_init(memcached_server_st *self, memcached_st *root,
   self->hostname[hostname.size]= 0;
 }
 
-static memcached_server_st *_server_create(memcached_server_st *self, const memcached_st *memc)
+static memcached_server_st *_server_create(memcached_server_st *self, const Memcached *memc)
 {
   if (self == NULL)
   {
@@ -105,7 +107,7 @@ static memcached_server_st *_server_create(memcached_server_st *self, const memc
   return self;
 }
 
-memcached_server_st *__server_create_with(memcached_st *memc,
+memcached_server_st *__server_create_with(Memcached *memc,
                                           memcached_server_st* allocated_instance,
                                           const memcached_string_t& hostname,
                                           const in_port_t port,
@@ -125,7 +127,7 @@ memcached_server_st *__server_create_with(memcached_st *memc,
     return NULL;
   }
 
-  _server_init(allocated_instance, const_cast<memcached_st *>(memc), hostname, port, weight, type);
+  _server_init(allocated_instance, const_cast<Memcached *>(memc), hostname, port, weight, type);
 
   return allocated_instance;
 }
@@ -191,16 +193,18 @@ uint32_t memcached_server_count(const memcached_st *self)
   return self->number_of_hosts;
 }
 
-const char *memcached_server_name(const memcached_server_instance_st self)
+const char *memcached_server_name(const memcached_instance_st * self)
 {
   WATCHPOINT_ASSERT(self);
-  if (self == NULL)
-    return NULL;
+  if (self)
+  {
+    return self->_hostname;
+  }
 
-  return self->hostname;
+  return NULL;
 }
 
-in_port_t memcached_server_port(const memcached_server_instance_st self)
+in_port_t memcached_server_port(const memcached_instance_st * self)
 {
   WATCHPOINT_ASSERT(self);
   if (self == NULL)
@@ -211,7 +215,25 @@ in_port_t memcached_server_port(const memcached_server_instance_st self)
   return self->port();
 }
 
-uint32_t memcached_server_response_count(const memcached_server_instance_st self)
+in_port_t memcached_server_srcport(const memcached_instance_st * self)
+{
+  WATCHPOINT_ASSERT(self);
+  if (self == NULL || self->fd == INVALID_SOCKET || (self->type != MEMCACHED_CONNECTION_TCP && self->type != MEMCACHED_CONNECTION_UDP))
+  {
+    return 0;
+  }
+
+  struct sockaddr_in sin;
+  socklen_t addrlen= sizeof(sin);
+  if (getsockname(self->fd, (struct sockaddr*)&sin, &addrlen) != -1)
+  {
+    return ntohs(sin.sin_port);
+  }
+
+  return -1;
+}
+
+uint32_t memcached_server_response_count(const memcached_instance_st * self)
 {
   WATCHPOINT_ASSERT(self);
   if (self == NULL)
@@ -222,7 +244,7 @@ uint32_t memcached_server_response_count(const memcached_server_instance_st self
   return self->cursor_active_;
 }
 
-const char *memcached_server_type(const memcached_server_instance_st ptr)
+const char *memcached_server_type(const memcached_instance_st * ptr)
 {
   if (ptr)
   {
@@ -242,7 +264,7 @@ const char *memcached_server_type(const memcached_server_instance_st ptr)
   return "UNKNOWN";
 }
 
-uint8_t memcached_server_major_version(const memcached_server_instance_st instance)
+uint8_t memcached_server_major_version(const memcached_instance_st * instance)
 {
   if (instance)
   {
@@ -252,7 +274,7 @@ uint8_t memcached_server_major_version(const memcached_server_instance_st instan
   return UINT8_MAX;
 }
 
-uint8_t memcached_server_minor_version(const memcached_server_instance_st instance)
+uint8_t memcached_server_minor_version(const memcached_instance_st * instance)
 {
   if (instance)
   {
@@ -262,7 +284,7 @@ uint8_t memcached_server_minor_version(const memcached_server_instance_st instan
   return UINT8_MAX;
 }
 
-uint8_t memcached_server_micro_version(const memcached_server_instance_st instance)
+uint8_t memcached_server_micro_version(const memcached_instance_st * instance)
 {
   if (instance)
   {
