@@ -35,7 +35,7 @@
  *
  */
 
-#include <config.h>
+#include <mem_config.h>
 #include <libtest/test.hpp>
 
 using namespace libtest;
@@ -48,14 +48,13 @@ using namespace libtest;
 static test_return_t pre_touch(memcached_st *memc)
 {
   test_compare(MEMCACHED_SUCCESS, memcached_version(memc));
-  test_skip(true, libmemcached_util_version_check(memc, 1, 4, 8));
+  test_skip(true, libmemcached_util_version_check(memc, 1, 4, 15));
 
   return TEST_SUCCESS;
 }
 
 test_return_t test_memcached_touch(memcached_st *memc)
 {
-
   test_skip(TEST_SUCCESS, pre_touch(memc));
 
   size_t len;
@@ -85,23 +84,20 @@ test_return_t test_memcached_touch(memcached_st *memc)
     free(value);
   }
 
-  test_compare(MEMCACHED_SUCCESS,
-               memcached_touch(memc, test_literal_param(__func__), 60 *60));
-
-  test_skip(false ,memcached_behavior_get(memc, MEMCACHED_BEHAVIOR_BINARY_PROTOCOL));
+  rc= memcached_touch(memc, test_literal_param(__func__), 60 *60);
+  ASSERT_EQ_(MEMCACHED_SUCCESS, rc, "%s", memcached_last_error_message(memc));
 
   rc= memcached_touch(memc, test_literal_param(__func__), 60 *60 *24 *60);
-  test_compare(MEMCACHED_SUCCESS, rc);
+  ASSERT_EQ_(MEMCACHED_SUCCESS, rc, "%s", memcached_last_error_message(memc));
 
-  test_compare(MEMCACHED_NOTFOUND,
-               memcached_exist(memc, test_literal_param(__func__)));
+  rc= memcached_exist(memc, test_literal_param(__func__));
+  ASSERT_EQ_(MEMCACHED_NOTFOUND, rc, "%s", memcached_last_error_message(memc));
 
   return TEST_SUCCESS;
 }
 
 test_return_t test_memcached_touch_by_key(memcached_st *memc)
 {
-
   test_skip(TEST_SUCCESS, pre_touch(memc));
 
   size_t len;
@@ -134,20 +130,20 @@ test_return_t test_memcached_touch_by_key(memcached_st *memc)
     free(value);
   }
 
-  test_compare(MEMCACHED_SUCCESS,
-               memcached_touch_by_key(memc,
-                                      test_literal_param("grouping_key"),
-                                      test_literal_param(__func__),
-                                      60 *60));
+  rc= memcached_touch_by_key(memc,
+                             test_literal_param("grouping_key"),
+                             test_literal_param(__func__),
+                             60 *60);
+  ASSERT_EQ_(MEMCACHED_SUCCESS, rc, "%s", memcached_last_error_message(memc));
 
-  test_skip(false ,memcached_behavior_get(memc, MEMCACHED_BEHAVIOR_BINARY_PROTOCOL));
-  test_compare(MEMCACHED_SUCCESS,
-               memcached_touch_by_key(memc,
-                                      test_literal_param("grouping_key"),
-                                      test_literal_param(__func__),
-                                      60 *60 *24 *60));
-  test_compare(MEMCACHED_NOTFOUND,
-               memcached_exist_by_key(memc, test_literal_param("grouping_key"),test_literal_param(__func__)));
+  rc= memcached_touch_by_key(memc,
+                             test_literal_param("grouping_key"),
+                             test_literal_param(__func__),
+                             60 *60 *24 *60);
+  ASSERT_EQ_(MEMCACHED_SUCCESS, rc, "%s", memcached_last_error_message(memc));
+
+  rc= memcached_exist_by_key(memc, test_literal_param("grouping_key"),test_literal_param(__func__));
+  ASSERT_EQ_(MEMCACHED_NOTFOUND, rc, "%s", memcached_last_error_message(memc));
 
   return TEST_SUCCESS;
 }
